%global __python /usr/bin/python2.7
%global pyver 27
%global pybindir /usr/lib/python2.7/bin
%global python_version 2.7
#global pyver %{nil}

%global hdf5version 1.8.12
%global hdf5numversion %(echo %{hdf5version} | tr -d .)

%global scalapackversion 2.0.2
%global scalapacknumversion %(echo %{scalapackversion} | tr -d .)


%if 0%{?openmpiversion:1}
#%{!?openmpiversion: %global openmpiversion 1.6.3}
%global openmpinumversion %( printf "%02d%02d%02d" `echo %{openmpiversion} | cut -d. -f1,2,3 --output-delimiter=" "` )
%global _prefix /usr/local/gpaw%{pyver}/hdf5-%{hdf5version}/openmpi-%{openmpiversion}/scalapack-%{scalapackversion}/%{version}
%global modulefile_path /usr/local/share/Modules/modulefiles/gpaw%{pyver}/hdf5-%{hdf5version}/openmpi-%{openmpiversion}/scalapack-%{scalapackversion}
%else
%global _prefix /usr/local/gpaw%{pyver}/hdf5-%{hdf5version}/%{version}
%global modulefile_path /usr/local/share/Modules/modulefiles/gpaw%{pyver}/hdf5-%{hdf5version}
%endif

%if 0%{?el6}
# Error: No Package found for mpich-devel on el6
ExcludeArch: ppc64
%endif

%if 0%{?fedora} >= 21 || 0%{?el7} || 0%{?el6}
%global blacs_libs 'mpiblacs'
%else
%global	blacs_libs 'mpiblacsCinit', 'mpiblacs'
%endif

%if 0%{?openmpiversion:1}
Name:			gpaw%{pyver}-hdf5-%{hdf5numversion}-openmpi%{openmpinumversion}-scalapack%{scalapacknumversion}
%else
Name:			gpaw%{pyver}-hdf5-%{hdf5numversion}
%endif
%global rname gpaw
Version:		1.0.0
Release:		3%{?dist}
Summary:		A grid-based real-space PAW method DFT code

License:		GPLv3+
URL:			https://wiki.fysik.dtu.dk/gpaw/
Source0:		https://wiki.fysik.dtu.dk/gpaw-files/%{rname}-%{version}.tar.gz


BuildRequires:		time

%ifarch %arm
BuildRequires:		atlas-devel
%else
%ifarch aarch64
BuildRequires:		atlas-devel
%else
%ifarch ppc64
BuildRequires:		atlas-devel
%else
BuildRequires:		openblas-devel
%endif
%endif
%endif

BuildRequires:		libxc-devel


BuildRequires:		python%{pyver}-devel
# no scipy on el6 ppc64
%if 0%{?el6}
%ifarch ppc64
%{nil}
%else
BuildRequires:		scipy%{pyver}
%endif
%endif

BuildRequires:		python%{pyver}-ase
Requires:		python%{pyver}-ase

BuildRequires:		%{rname}-setups
Requires:		%{rname}-setups

Requires:		tar
%if 0%{?openmpiversion:1}
BuildRequires:		openssh-clients
BuildRequires:		openmpi%{openmpinumversion}-gcc-devel
BuildRequires:		scalapack%{scalapacknumversion}-openmpi%{openmpinumversion}-gcc-devel
BuildRequires:          hdf5-%{hdf5numversion}-gcc-openmpi%{openmpinumversion}-devel
%else
BuildRequires:          hdf5-%{hdf5numversion}-gcc-devel
%endif

%description
GPAW is a density-functional theory (DFT) Python code based
on the projector-augmented wave (PAW) method and the
atomic simulation environment (ASE). It uses real-space uniform grids and
multigrid methods or atom-centered basis-functions.
%if 0%{?openmpiversion:1}
This version was compiled with openmpi %{openmpiversion} and scalapack %{scalapackversion}
%endif


%prep
%setup -q -n %{rname}-%{version}

# fail with old scalapack: disable - some features are untested!
sed -i "s|'parallel/ut_hsblacs\.py',|#'parallel/ut_hsblacs.py', # disabled by rpmbuild|" gpaw/test/__init__.py
sed -i "s|'pw/fulldiagk\.py',|#'pw/fulldiagk.py',  # disabled by rpmbuild|" gpaw/test/__init__.py
sed -i "s|'parallel/lcao_complicated\.py',|#'parallel/lcao_complicated.py',  # disabled by rpmbuild|" gpaw/test/__init__.py
# hangs on Rawhide (21) on 4 cores
sed -i "s|'pw/mgo_hybrids\.py',|#'pw/mgo_hybrids.py',  # disabled by rpmbuild|" gpaw/test/__init__.py
# http://listserv.fysik.dtu.dk/pipermail/gpaw-developers/2014-April/004501.html
sed -i "s|'excited_state\.py',|#'excited_state.py',  # disabled by rpmbuild|" gpaw/test/__init__.py

# Many other tests fail with numpy 1.4.1 on el6:
# FloatingPointError: invalid value encountered in isfinite
# https://github.com/numpy/numpy/issues/2169


%build
. /etc/profile.d/modules.sh

# To avoid replicated code define a macro
%global dobuild() \
CFLAGS="$RPM_OPT_FLAGS" %{__python} setup.py build

echo "extra_compile_args += ['-fPIC']" >> customize.py
# enable hdf5
sed -i "s/.*hdf5 =.*/hdf5 = True; libraries += ['hdf5']/" customize.py
%if 0%{?openmpiversion:1}
# build openmpi version
# enable scalapack
sed -i 's/scalapack =.*/scalapack = True/' customize.py
sed -i "s/\['scalapack'\]/[%{blacs_libs}, 'scalapack']/" customize.py
# enable parallel hdf5
#sed -i "s|.*hdf5 =.*|hdf5 = True; import os; extra_link_args += [os.path.join(os.environ['MPI_LIB'], 'libhdf5.so')]|" customize.py
# force mpicc
sed -i 's/#compiler =.*/compiler = "mpicc"/' customize.py
module load python hdf5/gcc/openmpi-%{openmpiversion}/%{hdf5version} openmpi
export CPATH=$CPATH:/usr/local/openmpi/%{openmpiversion}/gcc/x86_64/include
%dobuild
module purge
%else
# build serial version
# disable scalapack
sed -i 's/scalapack =.*/scalapack = False/' customize.py
module load python hdf5/gcc/%{hdf5version}
%dobuild
module purge
%endif

%install
. /etc/profile.d/modules.sh

CFLAGS="$RPM_OPT_FLAGS" %{__python} setup.py install --skip-build \
	--root $RPM_BUILD_ROOT

# Fedora complains about permissions of _gpaw.so
#chmod 0755 $RPM_BUILD_ROOT%{python_sitearch}/_%{rname}.so
#chmod 0755 $RPM_BUILD_ROOT%{python_sitearch}/_%{rname}_hdf5.so

mkdir -p $RPM_BUILD_ROOT%{_prefix}
mv $RPM_BUILD_ROOT/usr/bin $RPM_BUILD_ROOT%{_bindir}
mv $RPM_BUILD_ROOT/usr/%{_lib} $RPM_BUILD_ROOT%{_libdir}

mkdir -p %{buildroot}/%{modulefile_path}
cat <<ENDCOREMODULE > %{buildroot}/%{modulefile_path}/%{version}
#%Module

# NOTE: This is an automatically-generated file!  (generated by the
# gpaw rpm).  Any changes made here will be lost a) if the RPM is
# uninstalled, or b) if the RPM is upgraded or uninstalled.

proc ModulesHelp { } {
   puts stderr "This module adds gpaw%{pyver} %{version} for hdf5 %{hdf5version} to python path"
}

module-whatis   "Sets up h5py%{pyver} %{version} for hdf5 %{hdf5version} to python path"

prepend-path PATH %{_bindir}
prepend-path PYTHONPATH "%{_libdir}/python%{python_version}/site-packages"
ENDCOREMODULE

%check
# skip for now, no time
%if 0
. /etc/profile.d/modules.sh

module load python
export NPROC=4 # test on 4 cores (scalapack test need that)

%global GPAW_PLATFORM %(%{__python} -c "from distutils import util, sysconfig; print util.get_platform()+'-'+sysconfig.get_python_version()")

export PYTHONPATH=`pwd`/build/lib.%{GPAW_PLATFORM};
export PATH=`pwd`/tools:${PATH};

# To avoid replicated code define a macro
%global docheck() \
time $GPAW_EXECUTABLE `which gpaw-test` --range='gemm_complex.py,xc.py' --debug 2>&1 | tee gpaw-test${NPROC}$MPI_SUFFIX.log

%if 0%{?openmpiversion:1}
# check openmpi version
module load hdf5/gcc/openmpi-%{openmpiversion}/%{hdf5version} openmpi
GPAW_EXECUTABLE="mpiexec -np ${NPROC} build$MPI_SUFFIX/bin.%{GPAW_PLATFORM}/%{rname}-python" %docheck
%else
module load hdf5
# check serial version
MPI_SUFFIX="" GPAW_EXECUTABLE="%{__python}" %docheck
%endif
%endif

%files
%defattr(-,root,root,-)
%doc LICENSE
%{_prefix}
%dir %{modulefile_path}
%{modulefile_path}/%{version}

%changelog
* Thu Dec 17 2015 Marcin Dulak <Marcin.Dulak@gmail.com> - 0.11.0.13004-18
- get rid of old mpich globals

* Mon Dec 14 2015 Yaakov Selkowitz <yselkowi@redhat.com> - 0.11.0.13004-17
- Fix build on aarch64 (#1291383)

* Wed Sep 16 2015 Orion Poplawski <orion@cora.nwra.com> - 0.11.0.13004-16
- Rebuild for openmpi 1.10.0

* Sun Jul 26 2015 Sandro Mani <manisandro@gmail.com> - 0.11.0.13004-15
- Rebuild for RPM MPI Requires Provides Change

* Wed Jul 22 2015 Marcin Dulak <Marcin.Dulak@gmail.com> - 0.11.0.13004-14
- upstream update
- files-attr

* Wed Jun 17 2015 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 0.10.0.11364-13
- Rebuilt for https://fedoraproject.org/wiki/Fedora_23_Mass_Rebuild

* Thu May 21 2015 Marcin Dulak <Marcin.Dulak@gmail.com> - 0.10.0.11364-12
- mpi versions Require gpaw

* Sun May 17 2015 Orion Poplawski <orion@cora.nwra.com> - 0.10.0.11364-11
- Rebuild for hdf5 1.8.15

* Sun May  3 2015 Zbigniew Jędrzejewski-Szmek <zbyszek@in.waw.pl> - 0.10.0.11364-10
- Rebuild for changed mpich

* Wed Jan 07 2015 Orion Poplawski <orion@cora.nwra.com> - 0.10.0.11364-9
- Rebuild for hdf5 1.8.14

* Thu Nov 20 2014 Marcin Dulak <Marcin.Dulak@gmail.com> - 0.10.0.11364-8
- new style of linking blacs on EL6

* Thu Oct 23 2014 Marcin Dulak <Marcin.Dulak@gmail.com> - 0.10.0.11364-7
- mpich version 3 in EL6
- use atlas on aarch64
- ppc64 on EL7

* Sat Aug 16 2014 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 0.10.0.11364-6
- Rebuilt for https://fedoraproject.org/wiki/Fedora_21_22_Mass_Rebuild

* Sat Jun 07 2014 Marcin Dulak <Marcin.Dulak@gmail.com> - 0.10.0.11364-5
- explicit Requires

* Sat Jun 07 2014 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 0.10.0.11364-4
- Rebuilt for https://fedoraproject.org/wiki/Fedora_21_Mass_Rebuild

* Thu May 22 2014 Marcin Dulak <Marcin.Dulak@gmail.com> 0.10.0.11364-3
- consistent usage of RPM_BUILD_ROOT/RPM_OPT_FLAGS
- hdf5 enabled
- blacs-2.0.2 on fedora >= 21

* Fri May 2 2014 Marcin Dulak <Marcin.Dulak@gmail.com> 0.10.0.11364-2
- %%arm and ppc64 added
- more explicit globs in %%files
- gcc BR removed
- permissions of _gpaw.so fixed

* Tue Apr 8 2014 Marcin Dulak <Marcin.Dulak@gmail.com> 0.10.0.11364-1
- initial version for Fedora
